"""
Means to run multiple processes for publishing.
"""

import time
from abc import ABC, abstractmethod
import multiprocessing as mp
import queue as qu
from .pubsub import Publisher

class PublisherStream(ABC):
    """
    A class managing a single publishing process.
    """

    def __init__(self, **kwargs):
        self._kwargs = kwargs
        self.queue = None

    def _setup(self, queue, **kwargs):
        self.queue = queue
        self.setup(**kwargs)

    def setup(self, **kwargs):
        pass

    def publish(self, message):
        self.queue.put(message)

    @abstractmethod
    def run(self):
        pass

class PublisherPool:

    def __init__(self, port, streams):
        self.publisher = Publisher(port)
        self.queue = mp.Queue()
        self.streams = streams

    def flow(self):
        while True:
            processes = []

            for stream in self.streams:
                process = mp.Process(
                    target=_publisher_target,
                    args=(self.queue, stream)
                )
                process.start()
                processes.append(process)

            while True:
                try:
                    message = self.queue.get(timeout=30)
                    self.publisher.publish(message)
                except qu.Empty:
                    pass
                if all((not p.is_alive for p in processes)):
                    break

            for process in processes:
                process.join()

            time.sleep(10)

def _publisher_target(queue, stream):
    stream._setup(queue, **stream._kwargs)
    stream.run()